@@ -11,11 +11,11 @@ HTML::EditableTable::Javascript
 
 =head1 VERSION
 
-Version 0.20
+Version 0.21
 
 =cut
 
-our $Version = '0.20';
+our $Version = '0.21';
 
 my $javascriptDisplayCount = 0;
 
@@ -67,6 +67,8 @@ sub htmlDisplay {
 
   # for multiple table case, only print these once
 
+  print "debug displayCount = $javascriptDisplayCount<br>";
+
   if (!$javascriptDisplayCount) {
 
     # table field-level javascript support
@@ -132,6 +134,13 @@ sub htmlDisplay {
   }
 }
 
+sub resetJavascriptDisplayCount {
+  my $self = shift;
+  
+  print "reseting display count<br>";
+  $javascriptDisplayCount = 0;
+}
+
 sub jsExpandText {
 
   my $self = shift;
